package ru.amse.silina.cat.ui;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JSeparator;
import javax.swing.KeyStroke;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.stream.XMLStreamException;

import org.xml.sax.SAXException;

import ru.amse.silina.cat.serialize.Deserializer;
import ru.amse.silina.cat.serialize.Serializer;
import ru.amse.silina.cat.text.IText;
import ru.amse.silina.cat.text.markers.impl.XMLDeadSpacesMarker;
import ru.amse.silina.cat.ui.fragmentspane.JFragmentsPane;

public class CATFrame extends JFrame {
	private IText myText;
	private boolean myIsChanged = false;
	private boolean myAddsTranslated = true;
	private File myFile = null;
	private JFragmentsPane myTarget;
	private JFragmentsPane mySource;
	
	
	public CATFrame(IText text) {
		super("CATUI");
		myText = text;
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setSize(800, 500);
		Container c = getContentPane();
		c.setLayout(new BoxLayout(c, BoxLayout.X_AXIS));
		
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		
		//Add file menu
		JMenu fileMenu = new JMenu("File");
		menuBar.add(fileMenu);
		fileMenu.add(new OpenAction(this, "Open..."));
		fileMenu.add(new SaveAction(this, "Save"));
		fileMenu.add(new SaveAsAction(this, "Save as..."));
		fileMenu.add(new JSeparator());
		fileMenu.add(new AbstractAction("Exit") {
			@Override
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		});
		
		JMenu editMenu = new JMenu("Edit");
		menuBar.add(editMenu);
		JMenuItem delete = new JMenuItem(new DeleteAction("Delete selected fragment"));
		delete.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
		editMenu.add(delete);
		JMenuItem markXML = new JMenuItem(new markXMLAction("Parse XML and add deadspaces"));
		editMenu.add(markXML);
		
		
		//Add fragment type menu
		JMenu typeMenu = new JMenu("New fragment type");
		editMenu.add(typeMenu);
		ButtonGroup bg = new ButtonGroup();
		JRadioButtonMenuItem selectTranslated = new JRadioButtonMenuItem(new UseTranslatedAction("Add translated fragment"));
		selectTranslated.setSelected(true);
		bg.add(selectTranslated);
		typeMenu.add(selectTranslated);
		JRadioButtonMenuItem selectDeadSpace = new JRadioButtonMenuItem(new UseDeadSpaceAction("Add dead space"));
		bg.add(selectDeadSpace);
		typeMenu.add(selectDeadSpace);
		
		
		myTarget = new JFragmentsPane(this, text, new TargetFragmentDataProvider(text));
		mySource = new JFragmentsPane(this, text, new SourceFragmentDataProvider(text));
		
		myTarget.addFragmentClickListener(new FragmentClickListener(mySource, myTarget));
		mySource.addFragmentClickListener(new FragmentClickListener(mySource, myTarget));
		mySource.addFragmentMarkListener(new FragmentMarkListener(mySource, myTarget, this));
		myTarget.addFragmentMarkListener(new FragmentMarkListener(mySource, myTarget, this));
		
		c.add(mySource);
		c.add(Box.createHorizontalStrut(12));
		c.add(myTarget);
	}
	
	public void setIsChanged(boolean isChanged) {
		myIsChanged = isChanged;
	}
	
	private class UseTranslatedAction extends AbstractAction {
		
		public UseTranslatedAction(String name) {
			super(name);
		}
		
		@Override
		public void actionPerformed(ActionEvent e) {
			myAddsTranslated = true;
		}
	}
	
	private class UseDeadSpaceAction extends AbstractAction {
		
		public UseDeadSpaceAction(String name) {
			super(name);
		}
		
		@Override
		public void actionPerformed(ActionEvent e) {
			myAddsTranslated = false;
		}
	}
	
	private class OpenAction extends AbstractAction {
		private CATFrame myFrame;
		
		public OpenAction(CATFrame frame, String name) {
			super(name);
			myFrame = frame;
		}
		
		@Override
	    public void actionPerformed(ActionEvent e) {
	    	int confirmed = JOptionPane.NO_OPTION;
	    	if (myIsChanged) {
	    		confirmed = JOptionPane.showConfirmDialog(myFrame, "Do you want to save your changes?");
	    	}
	    	JFileChooser fc = new JFileChooser();
	    	if (confirmed == JOptionPane.YES_OPTION) {
	    		saveAs(myFrame, fc);
	    	} 
	    	if (confirmed != JOptionPane.CANCEL_OPTION) {
		    	int res = fc.showOpenDialog(myFrame);
		    	if (res == JFileChooser.APPROVE_OPTION) {
		    		Deserializer deserializer = new Deserializer();
		    		myFile = fc.getSelectedFile();
					try {
						myText = deserializer.deserialize(new FileInputStream(myFile));
						mySource.loadText(myText, new SourceFragmentDataProvider(myText));
						myTarget.loadText(myText, new TargetFragmentDataProvider(myText));
						myIsChanged = false;
					} catch (FileNotFoundException e1) {
						JOptionPane.showMessageDialog(myFrame, "Can't open file: " + e1.getMessage());
					} catch (ParserConfigurationException e1) {
						JOptionPane.showMessageDialog(myFrame, "Can't read XML: " + e1.getMessage());
					} catch (SAXException e1) {
						JOptionPane.showMessageDialog(myFrame, "Can't read XML: " + e1.getMessage());
					} catch (IOException e1) {
						JOptionPane.showMessageDialog(myFrame, "Can't open file: " + e1.getMessage());
					}
		    	}
	    		
	    	}
	    }
	};
	
	private class SaveAsAction extends AbstractAction {
		private CATFrame myFrame;
		
		public SaveAsAction(CATFrame frame, String name) {
			super(name);
			myFrame = frame;
		}
		
		@Override
		public void actionPerformed(ActionEvent e) {
			JFileChooser fc = new JFileChooser();
			saveAs(myFrame, fc);
		}
	}
	
	private class SaveAction extends AbstractAction {
		private CATFrame myFrame;
		
		public SaveAction(CATFrame frame, String name) {
			super(name);
			myFrame = frame;
		}
		
		@Override
		public void actionPerformed(ActionEvent e) {
			if (myFile == null) {
				JFileChooser fc = new JFileChooser();
				saveAs(myFrame, fc);
			} else {
				Serializer serializer = new Serializer();
				try {
					serializer.serialize(myText, new FileOutputStream(myFile));
				} catch (FileNotFoundException e1) {
					JOptionPane.showMessageDialog(myFrame, "Can't write to file: " + e1.getMessage());
				} catch (XMLStreamException e1) {
					JOptionPane.showMessageDialog(myFrame, "Can't write to file: " + e1.getMessage());
				}
			}
		}
	}

	public boolean addsTranslated() {
		return myAddsTranslated;
	}
	
	private void saveAs(CATFrame myFrame, JFileChooser fc) {
		int res = fc.showSaveDialog(myFrame);
		if (res == JFileChooser.APPROVE_OPTION) {
			Serializer serializer = new Serializer();
				myFile = fc.getSelectedFile();
				try {
					serializer.serialize(myText, new FileOutputStream(myFile));
					myIsChanged = false;
				} catch (FileNotFoundException e1) {
					JOptionPane.showMessageDialog(myFrame, "Can't write to file: " + e1.getMessage());
				} catch (XMLStreamException e1) {
					JOptionPane.showMessageDialog(myFrame, "Can't write to file: " + e1.getMessage());
				}
		}
	}

	private class DeleteAction extends AbstractAction {
		
		public DeleteAction(String name) {
			super(name);
		}
		
		@Override
		public void actionPerformed(ActionEvent e) {
			myText.removeFragment(mySource.getSelectedFragment());
			mySource.setSelectedFragment(null);
			myTarget.getFragmentDataProvider().recalculateFragmentPositions();
			myIsChanged = true;
			repaint();
		}
	}
	
	private class markXMLAction extends AbstractAction {
		
		public markXMLAction(String name) {
			super(name);
		}
		
		@Override
		public void actionPerformed(ActionEvent e) {
			XMLDeadSpacesMarker marker = new XMLDeadSpacesMarker(myText);
			marker.markFragments();
			myTarget.getFragmentDataProvider().recalculateFragmentPositions();
			myIsChanged = true;
			repaint();
		}
	}
}
