package ru.amse.silina.cat.ui;

import java.util.List;

import ru.amse.silina.cat.text.IFragment;
import ru.amse.silina.cat.text.IText;
import ru.amse.silina.cat.text.ITranslatedFragment;
import ru.amse.silina.cat.ui.fragmentspane.IFragmentDataProvider;

public class TargetFragmentDataProvider implements IFragmentDataProvider {
	private IText myText;
	private int[] myStarts;
	private int[] myEnds;
	
	public TargetFragmentDataProvider(IText text) {
		myText = text;
		recalculateFragmentPositions();
	}
	
	@Override
	public String getText() {
		return myText.getTargetText();
	}
	
	@Override
	public int getStart(IFragment fragment) {
		int indexOf = myText.getFragments().indexOf(fragment);
		return myStarts[indexOf];
	}

	@Override
	public int getEnd(IFragment fragment) {
		int indexOf = myText.getFragments().indexOf(fragment);
		return myEnds[indexOf];
	}
	
	@Override
	public final void recalculateFragmentPositions() {
		List<IFragment> fragments = myText.getFragments();
		int count = 0;
		int[] starts = new int[fragments.size()];
		int[] ends = new int[fragments.size()];
		int shift = 0;
		for (IFragment f : fragments) {
			starts[count] = f.getStart() + shift;
			if (f instanceof ITranslatedFragment) {
				shift += ((ITranslatedFragment)f).getTranslation().length() - (f.getEnd() - f.getStart()) - 1;
			} 
			ends[count] = f.getEnd() + shift; 
			count++;
		}
		myStarts = starts;
		myEnds = ends;
	}
	
	
}
