package ru.amse.silina.cat.ui.fragmentspane;

import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import ru.amse.silina.cat.text.IFragment;

public class UICalculator implements IUICalculator {
	private JFragmentsPane myPane;
	
	
	public UICalculator(JFragmentsPane pane) {
		myPane = pane;
	}
	
	public int getLineByOffset(List<Integer> firstInLine, int offset) {
		int size = firstInLine.size();
		int lineCount = size ;
		for (ListIterator<Integer> iterator = firstInLine.listIterator(size); iterator.hasPrevious();) {
			int line = iterator.previous();
			lineCount--;
			if (line <= offset) {
				return lineCount;
			}
		}
		return 0;
	}
	
	public List<Integer> buildFirstInLine(Graphics g) {
		String outputText = myPane.getFragmentDataProvider().getText();
		List<Integer> firstInLine = new ArrayList<Integer>();
		firstInLine.add(0, 0);
		int lineIndex = 0;
		int length = outputText.length();
		for (int i = 1; i < length; i++) {
			if (outputText.charAt(i) == '\n') {
				lineIndex++;
				firstInLine.add(i + 1);
			} else if (outputText.charAt(i) == ' ') {
				String word;
				if (outputText.indexOf(' ', i + 1) != -1) {
					word = outputText.substring(i, outputText.indexOf(' ', i + 1) + 1);
				} else {
					word = outputText.substring(i);
				}
				String prefix = outputText.substring(firstInLine.get(lineIndex), i);
				if (g.getFontMetrics().stringWidth(word) > myPane.getBounds().getWidth() - g.getFontMetrics().stringWidth(prefix)) {
					lineIndex++;
					firstInLine.add(i + 1);
				}
			}
		}
		return firstInLine;
	}
	
	public IFragment seekFragment(int x, int y, Graphics g) {
		int symbol = getOffsetByCoordinates(x, y, g);
		if (symbol == -1) {
			return null;
		}
		for (ListIterator<IFragment> i = myPane.getText().getFragments().listIterator(myPane.getText().getFragments().size()); i.hasPrevious(); ) {
			IFragment fragment = i.previous();
			if (symbol >= myPane.getFragmentDataProvider().getStart(fragment)) {
				if (symbol <= myPane.getFragmentDataProvider().getEnd(fragment)) {
					return fragment;
				} else {
					return null;
				}
			}
		}
		return null;
	}
	
	public int seekClosestFragmentOffsetForward(int offset) {
		List<IFragment> fragmentsList = myPane.getText().getFragments();
			IFragmentDataProvider dataProvider = myPane.getFragmentDataProvider();
			for (IFragment fragment : fragmentsList) {
				int start = dataProvider.getStart(fragment);
				if (start > offset) {
					return start;
				}
			}
			return dataProvider.getText().length();
	}
			
			
	public int seekClosestFragmentOffsetBack(int offset) {
		List<IFragment> fragmentsList = myPane.getText().getFragments();
		for (ListIterator<IFragment> i = fragmentsList.listIterator(fragmentsList.size()); i.hasPrevious(); ) {
			IFragment fragment = i.previous();
			int end = myPane.getFragmentDataProvider().getEnd(fragment);
			if (end < offset) {
				return end;
			}
		}
		return -1;
	}

	public int getOffsetByCoordinates(int x, int y, Graphics g) {
		List<Integer> firstInLine = myPane.getFirstInLine();
		int line = seekLine(y, g);
		String output = myPane.getFragmentDataProvider().getText();
		if (line >= firstInLine.size()) {
			return output.length() - 1;
		}
		if (line < 0) {
			return 0;
		}
		int first = firstInLine.get(line);
		int last = first;
		while ((last < output.length()) && (g.getFontMetrics().stringWidth(output.substring(first, last)) < x)) {
			last++;
		}
		int symbol = last - 1;
		return symbol;
	}

	private int seekLine(int y, Graphics g) {
		int lineHeight = g.getFontMetrics().getHeight();
		return (y - JFragmentsPane.SHIFT) / lineHeight;
	}
	
	
}
